//: connection/DBconnect.java
package connection;
import java.util.ArrayList;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

import java.util.Random;

/** Class connects into DB with English words
 *
 * @author Sokunova Mariya
 */
public class DBconnect implements DataConnection{

    private Connection conn = null;
    /**Method  establishes connection into mysql DB
     *
     * @param userName login
     * @param password passwd
     * @param dbname database for connect
     * @throws java.lang.Exception
     */
   
    public void setConnection(String userName,String password, String dbname) throws Exception{
        Connection connection = null;
        //Load Driver
        Class.forName ("com.mysql.jdbc.Driver").newInstance();
        //Establish the connection
        String url = "jdbc:mysql://localhost/"+dbname;
        connection = DriverManager.getConnection(url, userName, password);
        conn = connection;

   }
    /**
     * Method  closes connection
     * @throws java.sql.SQLException
     */
    public void closeConnection() throws SQLException{
        conn.close();

    }
    /**
     * Method prints SQLException in usability form
     * @param ex SQLException
     * @param methodName method which has thrown an exception
     */
    public static void printSQLException(SQLException ex, String methodName){
          System.err.println("Method: " + methodName);
          System.err.println("SQLState: " + ((SQLException)ex).getSQLState());
          System.err.println("Error Code: " + ((SQLException)ex).getErrorCode());
          System.err.println("Message: " + ((SQLException)ex).getMessage());
    }
    

    /** Method returns english word by it id
     *
     * @param id `id` in table english
     * @return `word` by `id`
     */
    private String getWordByID(Integer id) throws SQLException {
        Statement state = null;
        String selectRequest = "SELECT word FROM english WHERE id="+id;
        String word ="";
        try {
            state = conn.createStatement();
            ResultSet result = state.executeQuery(selectRequest);
            if(result.first()){
                word=result.getString("word");
            }
            

        }
        catch(SQLException ex)
        {
          printSQLException(ex,"getWordByID");            

        }
        finally{
            
                state.close();
                return word;

        }

    }
    
   
    
    
    /**
     * Method returns translate for English Word
     * @param engWord word for translating
     * @return List russian word
     */
    private ArrayList<String> getEngWordTranslate(String engWord) throws SQLException{
        ArrayList<String> rusWords = new ArrayList<String>();
        Statement state = null;
        String selectRequest = "select english.word,russian.word AS rus_word " +
                               "from russian inner join (translate inner join english " +
                               "on (translate.id_eng = english.id)) on " +
                               "russian.id = translate.id_rus " +
                               "where english.word like '" + engWord +"'";


        try {
            state = conn.createStatement();
            ResultSet result = state.executeQuery(selectRequest);
            while(result.next()){
                rusWords.add(result.getString("rus_word"));
            }


        } catch (SQLException ex) {
            printSQLException(ex,"getEngWordTranslate");
        }
        finally{
            state.close();
            return rusWords;
        }


    }

    /** Method returns random id in english table
     * 
     * @return random int `id`
     */
    private Integer getRandomEnglishID() throws SQLException{
        Statement state =null;
        String selectRequest = "SELECT id FROM english";
        ArrayList<Integer> ids= new ArrayList<Integer>();
        Integer id=0;
        try{
            state = conn.createStatement();
            ResultSet result= state.executeQuery(selectRequest);
            while(result.next()){
                ids.add(result.getInt("id"));
            }
            Random generator = new Random();
            int index = generator.nextInt ( ids.size() );
            if( index >-1 ) {
                id = ids.get(index);
            }
            
        }
        catch(SQLException ex){

            printSQLException(ex,"getRandomEnglishID");
        }
        finally{
            state.close();
            return id;

        }
        
    }
    /** Method returns random English word from DB
     *
     * @return english word or none
     */
    public String returnRandomWord(){
        String str="";
        try {

            str = getWordByID(getRandomEnglishID());
            
        }
        catch(SQLException ex){
            printSQLException(ex,"returnRandomWord");

        }
        return str;
    }
    /**
     * Method returns translates for an english word
     * @param englishWord word for translating
     * @return List of russian words
     */
    public ArrayList<String> returnAnswersByWord(String englishWord){
        ArrayList<String> answersList = new ArrayList<String>();
        try{
            answersList = getEngWordTranslate(englishWord);
        }
        catch(SQLException ex){
            printSQLException(ex,"returnAnswersByWord");


        }

        return answersList;

    }


}
